#include <signal.h>
#include "comp_s.hh"
#include "mon_c.hh"


class TestComputeServer: public ComputeServer_impl
{
    public:
	TestComputeServer() {}
	~TestComputeServer() {}

	CORBA::Float add(CORBA::Float a, CORBA::Float b) {
		cout << "TestComputeServer:: Adding " << a << " and "
			<< b << endl;
		return (a + b);
	}
};

static Monitor *monitor = NULL;

void sighandler(int)
{
	if (monitor )
		monitor->_release();
	exit(0);
}

int main()
{
	signal(SIGINT, sighandler);
        CORBA::Environment env;

	// Set scope to local so that all Object Implementations created 
	// from now on will be Transient Objects
	CORBA::BOA::scope(CORBA::BOA::SCOPE_LOCAL);

	//  Create a Transient ComputeServer implementation
	TestComputeServer server;

	// Bind to Monitor 
        monitor = Monitor::_bind(env);
        if ( env.check_exception() ) {
                cout << "Unable to bind to Monitor" << endl;
                return 1;
        }
 
	// Pass the transient ComputeServer Object to Monitor
        monitor->register_server(&server, env);
        if ( env.check_exception() ) {
                cout << "Error registering server" << endl;
                cout << env;
                return 1;
        }

	
	CORBA::BOA::impl_is_ready();
	return 0;
}

