#include <signal.h>
#include "perf_s.hh"

int seq_size = 100;

class OnewayServer: public TestOneway_impl
{
    public:
	OnewayServer() {};
	~OnewayServer() {}

	void test_no_param() {}
	void test_prim_args(CORBA::Short, CORBA::Long, CORBA::Float, 
		CORBA::Double, CORBA::Char, const CORBA::String&) {}
	void test_struct(const PerfStruct& ) {}
	void test_prim_seq(const shortSeq&, const longSeq&, const floatSeq&,
		const doubleSeq&, const charSeq&, const stringSeq& ) {}
	void test_struct_seq(const structSeq& ) {}
	void test_struct_array(const structArray& ) {}
};

class ReqReplyServer: public TestReqReply_impl
{
    private:
	shortSeq	*_shortSeq;
	longSeq		*_longSeq;
	floatSeq	*_floatSeq;
	doubleSeq	*_doubleSeq;
	charSeq		*_charSeq;
	stringSeq	*_stringSeq;
	structSeq	*_structSeq;
	structArray	*_structArray;

    public:
	ReqReplyServer();
	~ReqReplyServer() { 
		_shortSeq->_release();
		_longSeq->_release();
		_floatSeq->_release();
		_doubleSeq->_release();
		_charSeq->_release();
		_stringSeq->_release();
		_structSeq->_release();
		_structArray->_release();
	}

	CORBA::Long test_prim_args(
			CORBA::Short, 
			CORBA::Long, 
			CORBA::Float, 
			CORBA::Double,
			CORBA::Char,
			const CORBA::String&, 
			CORBA::Short& ,
			CORBA::Long& ,
			CORBA::Float& ,
			CORBA::Double& ,
			CORBA::Char& ,
			CORBA::String& ,
			CORBA::Short& out_short,
			CORBA::Long& out_long,
			CORBA::Float& out_float,
			CORBA::Double& out_double,
			CORBA::Char& out_char,
			CORBA::String *&out_string) {
		
		out_short = 2;
		out_long = 2;
		out_float = 2.0;
		out_double = 2.0;
		out_char = 2;
		out_string = new CORBA::String("string");
	
		return 1;
	}

	CORBA::Long test_struct_args(const PerfStruct&, 
			PerfStruct&, 
			PerfStruct *&out_struct)
	{
		out_struct = new PerfStruct;
		return 1;
	}

	CORBA::Long test_prim_seq(
			const shortSeq&,
			const longSeq&,
			const floatSeq&,
			const doubleSeq&,
			const charSeq&,
			const stringSeq&,

			shortSeq&,
			longSeq&,
			floatSeq&,
			doubleSeq&,
			charSeq&,
			stringSeq&,

			shortSeq *& out_short,
			longSeq *& out_long,
			floatSeq *& out_float,
			doubleSeq *& out_double,
			charSeq *& out_char,
			stringSeq *& out_string) {
		out_short = _shortSeq;
		out_long = _longSeq;
		out_float = _floatSeq;
		out_double = _doubleSeq;
		out_char = _charSeq;
		out_string = _stringSeq;

		// ORB core releases return/ out parameters
		// Objects are duplicated so that they are not deleted
		_shortSeq->_duplicate();
		_longSeq->_duplicate();
		_floatSeq->_duplicate();
		_doubleSeq->_duplicate();
		_charSeq->_duplicate();
		_stringSeq->_duplicate();

		return 1;
	}

	CORBA::Long test_struct_seq(const structSeq&, structSeq&,
			structSeq *& out_seq) {
		out_seq = _structSeq;
		_structSeq->_duplicate();
		return 1;
	}

	CORBA::Long test_struct_array(const structArray&, structArray&,
			structArray *& out_array) {
		out_array = _structArray;
		_structArray->_duplicate();
		return 1;
	}
};


ReqReplyServer::ReqReplyServer()
{
	_shortSeq = new shortSeq;
	_shortSeq->_duplicate();
	_longSeq = new longSeq;
	_longSeq->_duplicate();
	_floatSeq = new floatSeq;
	_floatSeq->_duplicate();
	_doubleSeq = new doubleSeq;
	_doubleSeq->_duplicate();
	_charSeq = new charSeq;
	_charSeq->_duplicate();
	_stringSeq = new stringSeq;
	_stringSeq->_duplicate();
	_structSeq = new structSeq;
	_structSeq->_duplicate();
	_structArray = new structArray;
	_structArray->_duplicate();
	for (int i=0; i< seq_size; i++) {
		_shortSeq->append(0);
		_longSeq->append(0);
		_floatSeq->append(0);
		_doubleSeq->append(0);
		_charSeq->append(0);
		_stringSeq->append(new CORBA::String("String"));
		_structSeq->append(new PerfStruct);
	}
}

void sighandler(int)
{
	exit(0);
}

int main()
{
	signal(SIGINT, sighandler);

	OnewayServer server;
	ReqReplyServer reqReply;
	CORBA::BOA::impl_is_ready();
	return 0;
}
