#include <stdlib.h>
#include <math.h>
#include <signal.h>
#include <dispatch/iohandle.h>
#include <dispatch/dispatch.h>
#include "feed_s.hh"

#define MAXRAND (pow(2,31)-1)
#define MYRAND(X) (int)rint(rand()/MAXRAND*X)


class FeedHandler: public Feed_impl, public IOHandler
{
    private:
	StockConsumerList	_assocList;

	void timerExpired(long, long);
    public:
	FeedHandler(const char *name) : Feed_impl(name) {}
	~FeedHandler() {}

	void	register_interest(StockConsumer *consumer, 
				const CORBA::String& stockName);
};


// Timer callback
void FeedHandler::timerExpired(long, long)
{
	CORBA::ULong num_stocks = _assocList.length();
	if ( num_stocks == 0 ) // No consumers/ no interest
		return;

	// Iterate through the assocList. Generate random prices and
	// send to consumers
	for (CORBA::ULong i= num_stocks-1; i < num_stocks; i--) {
		StockQuote quote;
		quote.stock_symbol() = _assocList[i].stock_name();
		quote.price(80.0 + rand()/MAXRAND * 40.0);
		quote.code() = MYRAND(80);
		ConsumerList& list = _assocList[i].consumers();
		CORBA::ULong num_consumers = list.length();
		for (CORBA::ULong j= num_consumers-1; j < num_consumers; j--) {
			StockConsumer *consumer = list[j];
			CORBA::Environment& env = consumer->_environment();
			consumer->receive(quote);
			if ( env.check_exception() )  // consumer must be down
				list.remove(j);		// Remove
		}
		// If no consumers, remove the entry from assocList
		if ( list.length() == 0) 
			_assocList.remove(i);
	}
	if ( _assocList.length() > 0) 
		Dispatcher::instance().startTimer(5, 0, this); // Set of timer
}

void FeedHandler::register_interest(StockConsumer *consumer, 
				const CORBA::String& stockName)
{
	cout << "Consumer registered interest in stock " << stockName << endl;

	CORBA::ULong num_stocks = _assocList.length();

	for (CORBA::ULong i=0; i< num_stocks; i++) {
		if ( _assocList[i].stock_name() == stockName ) {
			_assocList[i].consumers().append(consumer);
			break;
		}
	}
	if ( i == num_stocks ) { // new stock
		StockConsumerAssoc *assoc = new StockConsumerAssoc;
		assoc->stock_name(stockName);
		assoc->consumers().append(consumer);
		_assocList.append(assoc);
	}
	if ( num_stocks == 0)
		Dispatcher::instance().startTimer(5, 0, this);
}

void sighandler(int)
{
	exit(0);
}
			
int main(int argc, char **argv)
{
	signal(SIGINT, sighandler);

	if ( argc < 2 ) {
		cout << "Usage: feedhdlr name" << endl;
		return 0;
	}
	
	FeedHandler handler(argv[1]);
	
	CORBA::BOA::impl_is_ready();
	return 0;
}
	
	
	
